<?php

/**
 * @file
 * OpenID Connect client for ClaveUnica.
 */

/**
 * Implements OpenID Connect Client plugin for ClaveUnica.
 */
class OpenIDConnectClientClaveUnica extends OpenIDConnectClientBase {

  const USER_INFO_RUN_KEY = 'RolUnico';

  const USER_INFO_RUN_KEY_NUMBER = 'numero';

  const USER_INFO_RUN_KEY_DV = 'DV';

  const USER_INFO_NAME_KEY_FIRST_NAME= 'nombres';

  const USER_INFO_NAME_KEY_LAST_NAME= 'apellidos';

  /**
   * @var array
   */
  protected $user_data;

  /**
   * {@inheritdoc}
   */
  public function getEndpoints() {
    return array(
      'authorization' => 'https://accounts.claveunica.gob.cl/openid/authorize/',
      'token' => 'https://accounts.claveunica.gob.cl/openid/token/',
      'userinfo' => 'https://accounts.claveunica.gob.cl/openid/userinfo/',
    );
  }

  /**
   * {@inheritdoc}
   */
  public function decodeIdToken($id_token) {
    $this->user_data = parent::decodeIdToken($id_token);

    return $this->user_data;
  }

  /**
   * {@inheritdoc}
   */
  public function authorize($scope = 'openid email') {
    parent::authorize('openid run name');
  }

  /**
   * {@inheritdoc}
   */
  public function retrieveUserInfo($access_token) {
    $userinfo = parent::retrieveUserInfo($access_token);

    if (!empty($userinfo)) {
      $userinfo['email'] = $this->generateTemporaryEmail($userinfo);
      $userinfo['preferred_username'] = $this->generatePreferredUsername($userinfo);
      $userinfo['family_name'] = $this->buildFamilyName($userinfo);
      $userinfo['given_name'] = $this->buildGivenName($userinfo);
      $userinfo['surname'] = $this->buildSurname($userinfo);
      $userinfo['second_surname'] = $this->buildSecondSurname($userinfo);
      $userinfo['run_complete'] = $this->buildRunComplete($userinfo);
      $userinfo['run_number'] = $this->buildRunNumber($userinfo);
      $userinfo['run_dv'] = $this->buildRunDv($userinfo);
      $userinfo['name'] = $this->buildFullName($userinfo);

      drupal_alter('openid_claveunica_post_retrieve_userinfo', $userinfo, $this->user_data);
    }

    return $userinfo;
  }

  /**
   * Generates a temporary email to be modified later by the user.
   *
   * @param array $userinfo
   *  The data retrieved from the provider.
   *
   * @return string
   *  The temporary email.
   */
  protected function generateTemporaryEmail(array $userinfo) {
    $run = $userinfo[self::USER_INFO_RUN_KEY][self::USER_INFO_RUN_KEY_NUMBER];
    return $run . OPENID_CLAVEUNICA_TEMPORARY_SUFFIX_EMAIL;
  }

  /**
   * Generate the preferred username using the run.
   *
   * @param array $userinfo
   *  The data retrieved from the provider.
   *
   * @return string
   *  The preferred username.
   */
  protected function generatePreferredUsername(array $userinfo) {
    return $userinfo[self::USER_INFO_RUN_KEY][self::USER_INFO_RUN_KEY_NUMBER];
  }

  /**
   * Build the surname.
   *
   * @param array $userinfo
   *  The data retrieved from the provider.
   *
   * @return string
   *  The surname.
   */
  protected function buildSurname(array $userinfo) {
    return isset($userinfo['name'][self::USER_INFO_NAME_KEY_LAST_NAME][0]) ? $userinfo['name'][self::USER_INFO_NAME_KEY_LAST_NAME][0] : '';
  }

  /**
   * Build the second surname.
   *
   * @param array $userinfo
   *  The data retrieved from the provider.
   *
   * @return string
   *  The second surname.
   */
  protected function buildSecondSurname(array $userinfo) {
    return isset($userinfo['name'][self::USER_INFO_NAME_KEY_LAST_NAME][1]) ? $userinfo['name'][self::USER_INFO_NAME_KEY_LAST_NAME][1] : '';
  }

  /**
   * Build the family name.
   *
   * @param array $userinfo
   *  The data retrieved from the provider.
   *
   * @return string
   *  The family name.
   */
  protected function buildFamilyName(array $userinfo) {
    return implode(' ', $userinfo['name'][self::USER_INFO_NAME_KEY_LAST_NAME]);
  }

  /**
   * Build the given name.
   *
   * @param array $userinfo
   *  The data retrieved from the provider.
   *
   * @return string
   *  The given name.
   */
  protected function buildGivenName(array $userinfo) {
    return implode(' ', $userinfo['name'][self::USER_INFO_NAME_KEY_FIRST_NAME]);
  }

  /**
   * Build the full name from given and family names.
   *
   * @param array $userinfo
   *  The data retrieved from the provider.
   *
   * @return string
   *  The full name.
   */
  protected function buildFullName(array $userinfo) {
    return implode(' ', array($userinfo['given_name'], $userinfo['family_name']));
  }

  /**
   * Build the run complete.
   *
   * @param array $userinfo
   *  The data retrieved from the provider.
   *
   * @return string
   *  The full run.
   */
  protected function buildRunComplete(array $userinfo) {
    return implode('-', array(
      $this->buildRunNumber($userinfo),
      $this->buildRunDv($userinfo),
    ));
  }

  /**
   * Extracts the <Run> number from userinfo.
   *
   * @param array $userinfo
   *  The data retrieved from the provider.
   * @return string
   *  The run number.
   */
  protected function buildRunNumber(array $userinfo) {
    return $userinfo[self::USER_INFO_RUN_KEY][self::USER_INFO_RUN_KEY_NUMBER];
  }

  /**
   * Extracts the check digit from the <Run>.
   *
   * @param array $userinfo
   *  The data retrieved from the provider.
   *
   * @return string
   *  The check digit.
   */
  protected function buildRunDv(array $userinfo) {
    return $userinfo[self::USER_INFO_RUN_KEY][self::USER_INFO_RUN_KEY_DV];
  }
}
